package ucl.salinization.context;

import gov.nasa.worldwind.geom.LatLon;
import gov.nasa.worldwind.geom.Position;
import gov.nasa.worldwind.render.Polyline;
import gov.nasa.worldwind.render.Renderable;
import gov.nasa.worldwind.render.SurfacePolygon;
import gov.nasa.worldwind.render.SurfacePolyline;
import gov.nasa.worldwind.render.UserFacingIcon;

import java.awt.Color;
import java.awt.Dimension;
import java.io.File;
import java.util.ArrayList;
import java.util.Collections;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.Set;

import org.geotools.referencing.CRS;
import org.geotools.referencing.factory.ReferencingFactory;
import org.geotools.referencing.operation.DefiningConversion;
import org.jscience.geography.coordinates.UTM;
import org.jscience.geography.coordinates.crs.ReferenceEllipsoid;
import org.opengis.parameter.ParameterValueGroup;
import org.opengis.referencing.crs.CRSFactory;
import org.opengis.referencing.crs.GeographicCRS;
import org.opengis.referencing.crs.ProjectedCRS;
import org.opengis.referencing.cs.CartesianCS;
import org.opengis.referencing.operation.Conversion;
import org.opengis.referencing.operation.MathTransformFactory;

import repast.simphony.context.Context;
import repast.simphony.engine.environment.RunEnvironment;
import repast.simphony.engine.environment.RunState;
import repast.simphony.parameter.DefaultParameterSchema;
import repast.simphony.random.RandomHelper;
import repast.simphony.space.gis.Geography;
import repast.simphony.space.graph.Network;
import ucl.salinization.data.ScenarioData;
import ucl.salinization.dataLoad.CSVLoader;
import ucl.salinization.dataLoad.ShapeFileLoader;
import ucl.salinization.model.landuse.HydroBehavior;
import ucl.salinization.model.physical.Field;
import ucl.salinization.model.physical.FieldSetting;
import ucl.salinization.model.physical.HydrologicalSetting;
import ucl.salinization.model.physical.HydrologicalUnit;
import ucl.salinization.model.physical.SalinizationModel;
import ucl.salinization.social.Community;
import ucl.salinization.social.Person;
import ucl.salinization.social.Sex;
import ucl.salinization.model.physical.RemoteSensingEstimator;

import cern.jet.random.Uniform;

import com.vividsolutions.jts.geom.Coordinate;
import com.vividsolutions.jts.geom.Geometry;
import com.vividsolutions.jts.geom.MultiPolygon;
import com.vividsolutions.jts.geom.Point;

/**
 * Class for creating a scenario for a simulation based on input files and xml data.
 * @author 
 */

public class ScenarioBuilder {

	
	private int hydroNumber;  //number to keep track of hydrological systems
//	private List<String>waterSourceNames;
	private HashMap<String, Map<Integer, double[]>> hydroSourceData;  //hydrological source data
	private List<Community> communities;  //list of communities in this scenario
	
	private HashMap<String,Community>communityHash=new HashMap<String,Community>();
	public ScenarioBuilder(){
		hydroNumber=0;
	//	waterSourceNames=new ArrayList<String>();
		Person.Id=0;
		
	}
	
	/**
	 * Method to build a population from initial scenario information
	 * @param sData the scenario data
	 * @param waterUseModel the water use model
	 * @param geography the geography object
	 * @return a list of communities
	 */
	public List<Community> buildPopulation(Context context, ScenarioData sData,Geography geography){
		List<Community> communities = new ArrayList<Community>();
		//load the file here
		if(sData.getCommunityFile().contains(".shp")){
			List<Map<String, Object>> communityData=ShapeFileLoader.shpLoader(new 
					File(sData.getCommunityFile()));
			Iterator<Map<String, Object>> ic = communityData.iterator();
			int i=0;
			while(ic.hasNext()){
				Map<String,Object> data = ic.next();
				String name = (String)data.get("NAME");
				String population = (String)data.get("POPULATION");
				int pop = Integer.parseInt(population);
				Geometry geo = (Geometry)data.get("the_geom");
				Community community = new Community(name,(Point)geo);
				communityHash.put(name, community);
				
				List<Person> firstHalf=createPeopleSpread
					(community,context,geography,pop,Sex.MALE,(Point)
					geo,15,45);
		
		//		List<Person> secondHalf=createPeopleSpread(
		//				community,context,geography,tenth/2,Sex.FEMALE,(Point)geo,
		//				0,70);
				community.setPopulation(pop);
				community.addAll(firstHalf);
		//		community.addAll(secondHalf);
				List<Person>peeps=new ArrayList<Person>();
				peeps.addAll(firstHalf);
		//		peeps.addAll(secondHalf);
				community.setPeople(peeps);
				geography.move(community, geo);
				communities.add(community);
				i++;
			}
		}
		this.communities=communities;
		return communities;
	}
	
	public FieldSetting buildFields(Context context, Geography geography,ScenarioData sd){
		FieldSetting fs = new FieldSetting();
		List<Map<String, Object>> data=ShapeFileLoader.shpLoader(new 
				File(sd.getFieldData()));
		
		Iterator<Map<String, Object>> ic = data.iterator();
		List<Field>fields = new ArrayList<Field>();
	
		while(ic.hasNext()){
			Map value = ic.next();
			
			MultiPolygon poly = (MultiPolygon)value.get("the_geom");
			double a=poly.getArea();
			
			String fname = (String)value.get("NAME_F");
			String name = (String)value.get("NAME");
		//	if(name.equals("Tell Midal Ihmud"))
		//		name="Tell Midar Ihmud";
			String type = (String)value.get("TYPE");
			String l=(String)value.get("LEACHING");
			String y=(String)value.get("YIELD");
			String cap=(String)value.get("CAP");
			String capSD=(String)value.get("CAP_SD");
			String salt=(String)value.get("SALT");
//			String sRemove=(String)value.get("REMOVE");
			String leachingS=(String)value.get("LEACHING_S");
			String number = (String) value.get("NUMBER");
			double iSaline=	(Double) value.get("I_SALINE");
			String wtSalt= (String)value.get("WT_SALINE");
			String depth= (String)value.get("SOIL_L");
//			String fieldStorage= (String)value.get("FW_STORE");
			double leachEffec=(Double)value.get("LEACH_E");
//			String percRate=(String)value.get("PERC_R");
			
			Community c = communityHash.get(name);
			Field f = new Field(poly);
			f.setName(fname);
			if(f.getId()%2==0)
				f.setFallow(true);
			else
				f.setFallow(false);
			if(c==null) {
				continue;
			}
			f.setLeachingFactor(Double.parseDouble(l));
			f.setCommunity(c);
			f.setYield(Double.parseDouble(y));
			if(c.getFields()==null){
				List<Field>ffs=new ArrayList<Field>();
				ffs.add(f);
				c.setFields(ffs);
			}
			else
				c.getFields().add(f);
			if(type.equals("1")){
				int t = Integer.parseInt(type);
				f.setFieldType(t);
			}
			else if(type.equals("0"))
				f.setFieldType(0);
			else
				f.setFieldType(-1);
			
			f.setSaltCompounds(Double.parseDouble(salt));
			f.setCapillarySalt(Double.parseDouble(cap));
			f.setCapillarySaltSD(Double.parseDouble(capSD));
//			f.setSaltRemovePercent(Double.parseDouble(sRemove));
			f.setId(Integer.parseInt(number));
			f.setInitialSalinization(iSaline);
//			f.setPercolationRate(Double.parseDouble(percRate));
			f.setLayerThickness(Double.parseDouble(depth));
//			f.setWaterStoredSaturation(Double.parseDouble(fieldStorage));
			f.setWaterTableSalt(Double.parseDouble(wtSalt));
			f.setLeachingEfficiency(leachEffec);
			f.setLeachingSTD(Double.parseDouble(leachingS));
			f.setInitialLeachingFactor(f.getLeachingFactor());
			f.setInitialEfficiency(leachEffec);
			f.setInitialStandardDeviation(Double.parseDouble(leachingS));
			f.setInitialRatioUptake(f.getRatioUptake());
			f.setInitialWaterTable(f.getWaterTableSalt());
			fields.add(f);
			context.add(f);
			geography.move(f, poly);
		}
		
		fs.setFields(fields);
		fs.addAll(fields);
		return fs;
	}
	
	public FieldSetting buildFieldColors(Context context, Geography geography,ScenarioData sd){
		FieldSetting fs = new FieldSetting();
		List<Map<String, Object>> data=ShapeFileLoader.shpLoader(new 
				File(sd.getFieldData()));
		
		Iterator<Map<String, Object>> ic = data.iterator();
		List<Field>fields = new ArrayList<Field>();
	
		while(ic.hasNext()){
			Map value = ic.next();
			
			MultiPolygon poly = (MultiPolygon)value.get("the_geom");
			
			double salt=(Double)value.get("FSalt");
			double yield=(Double)value.get("FYield");
			int type=(Integer)value.get("FType");
			int id = (Integer)value.get("FNumber");
			
			Community c = communityHash.values().iterator().next();
			Field f = new Field(poly);
			if(f.getId()%2==0)
				f.setFallow(true);
			else
				f.setFallow(false);
			if(c==null) {
				continue;
			}
			f.setId(id);
			f.setCommunity(c);
			f.setFallowYears(yield);
			f.setFieldType(type);
			if(c.getFields()==null){
				List<Field>ffs=new ArrayList<Field>();
				ffs.add(f);
				c.setFields(ffs);
			}
			else
				c.getFields().add(f);
			
				
			
			f.setSaltCompounds(salt);
			
			fields.add(f);
			context.add(f);
			geography.move(f, poly);
		}
		
		fs.setFields(fields);
		fs.addAll(fields);
		return fs;
	}
	
	private void setupSocialModels(Community community, ScenarioData sd){
		if(sd.getWaterUseModel()!=null) {
	//		WaterUseModel wModel=WaterUseBehaviors.CreateWaterUseModel(sd.getWaterUseModel(),sd,community);
	//		community.setWaterUseModel(wModel);
		}
	}
		
	/**
	 * Method to create a population for a town based on file data
	 * @param data file data 
	 * @param community the community object
	 * @return a list of people
	 */
	public List<Person> createPeople(Map<String,Object> data,Geography geography,Community community){
		List<Person>people = new ArrayList<Person>();
		long id=0;
		Geometry geo = (Geometry)data.get("the_geom");
		Point point=(Point)geo;
		Set<String> keys = data.keySet();
		Iterator<String> ik = keys.iterator();
		while(ik.hasNext()){
			String key = ik.next();
			int popCategory=0;
			try{
				popCategory=((Double)data.get(key)).intValue();
			}
			catch(ClassCastException e){
				
			}
			String[] split=null;
			if(key.startsWith("MA")|| key.startsWith("FE")) {
				Sex sex =null;
				if(key.startsWith("MA"))
						sex=Sex.MALE;
				else
					sex=Sex.FEMALE;
				if(key.contains("_"))
					split=key.split("_");
				if(split==null){
					if(key.endsWith("OVER")){
						List<Person> cPeople=createPeopleSpread(community,null,
								geography,popCategory,sex,point,85,100);
						people.addAll(cPeople);
						id+=cPeople.size();
						continue;
					}
					for(int i=0; i < popCategory;i++){
						Person person = new Person(community, Integer.parseInt(key.substring(key.length()-1)),sex,point);
						people.add(person);
					}
				}
				else {
					if(split.length==1){
						for(int i=0; i < popCategory;i++){
							int age=Integer.parseInt(split[0].substring(split[0].length()-1))-1;
							Person person = new Person(community,age,sex,point);
							people.add(person);
						}
					}
					else{
						int ageFrom=Integer.parseInt(split[0].substring(split[0].length()-2));
						int ageTo=Integer.parseInt(split[1].substring(split[1].length()-2));
						List<Person> cPeople=createPeopleSpread(community,null,
								geography,popCategory,sex,point,ageFrom,ageTo);
						people.addAll(cPeople);
					}
				}	
			}
		}
		return people;
	} 
	
	/**
	 * Method to create people based on population data
	 * @param community the person's community
	 * @param number 
	 * @param sex sex type
	 * @param point the location
	 * @param ageFrom beginning age
	 * @param ageTo end age
	 * @return
	 */
	public List<Person> createPeopleSpread(Community community,Context context, 
			Geography geography,int number,Sex sex,Point point,int ageFrom,int ageTo){
		List<Person>people = new ArrayList<Person>();
		Uniform uniform=RandomHelper.createUniform();
		
		for(int i=0; i < number; i++){
			int age=uniform.nextIntFromTo(ageFrom, ageTo);
			Person person = new Person(community,age,sex,point);
			geography.move(person, point);
			context.add(person);
			people.add(person);
		}
		
		return people;
	}
	
	/**
	 * Method to load file data describing community characteristics
	 * @param community the community
	 * @param data the data for the scenario
	 */
	public void addCommunityDetails(Community community,ScenarioData data){
		double fertilityRate=0d;
		double migrationRate=0d;
		List<Map>communityData=CSVLoader.loadCSVStatic(data.getDemographicFile());
		Iterator<Map> ic = communityData.iterator();
		double[] rates = new double[101];
		while(ic.hasNext()){
			Map<String,Object> dataPoint = ic.next();
			if(!((String)dataPoint.get("Geography")).trim().equals(community.getId()))
				continue;
			Iterator<String> si = dataPoint.keySet().iterator();
			while(si.hasNext()){
				String key = si.next();
				if(key.startsWith("Death Rate")){
					double rate = Double.valueOf((String)dataPoint.get(key));
					String[] split = key.split(" ");
					String last = split[3];
					if(last.contains("-")) {
						String[] splitLast=last.split("-");
						int value1 = Integer.parseInt(splitLast[0]);
						int value2 = Integer.parseInt(splitLast[1]);
						rates=createRates(value1,value2,rates,rate,100000);
					}
					else if(last.contains("+")){
						String modified=last.substring(0, last.length()-1);
						int value=Integer.parseInt(modified);
						rates=createRates(value,100,rates,rate,100000);
					}
				}
			}
			fertilityRate=(Double)Double.valueOf((String)dataPoint.get("Birth Rate"));
			migrationRate=(Double)Double.valueOf((String)dataPoint.get("Migration"));
		}
		
	}
	
	/**
	 * Function for creating demographic rates
	 * @param value1 start value
	 * @param value2 end value
	 * @param rates an array of rates for the values
	 * @param rate the numerator to divide and get the rate
	 * @param denom the denominator used for the rate
	 * @return
	 */
	private double[] createRates(int value1, int value2, double[] rates,double rate,double denom){
		for(int i=value1; i<=value2;i++){
			rates[i]=rate/denom;
		}
		return rates;
	}
	
/*	public void projection(CRS input){
		MathTransformFactory mtFactory = ReferencingFinder.getMathTransformFactory(null);
		CRSFactory crsFactory = ReferencingFactoryFinder.getCRSFactory(null);
	    
	    GeographicCRS geoCRS = org.geotools.referencing.crs.DefaultGeographicCRS.WGS84;
	    CartesianCS cartCS = org.geotools.referencing.cs.DefaultCartesianCS.GENERIC_2D;
	    
	    ParameterValueGroup parameters =  mtFactory.getDefaultParameters("Transverse_Mercator");
	    parameters.parameter("central_meridian").setValue(-111.0);
	    parameters.parameter("latitude_of_origin").setValue(0.0);
	    parameters.parameter("scale_factor").setValue(0.9996);
	    parameters.parameter("false_easting").setValue(500000.0);
	    parameters.parameter("false_northing").setValue(0.0);
	    Conversion conversion = new DefiningConversion("Transverse_Mercator", parameters);
	    
	    Map<String, ?> properties = Collections.singletonMap("name", "WGS 84 / UTM Zone 38N");
	    ProjectedCRS projCRS = crsFactory.createProjectedCRS(properties, geoCRS, conversion, cartCS);
	} */
	
	/**
	 * Method to build hydrology for the scenario
	 * @param sd the scenario data containing data references
	 * @param hydrologicalModel the hydrology model to use
	 * @param geography the geography context
	 * @return
	 */
	public HydrologicalSetting buildHydrology(Context context,Geography geography,ScenarioData sd, String hydrologicalModel){
		
		HydrologicalSetting hs = new HydrologicalSetting(sd,hydroNumber,hydrologicalModel);
		Map<String,Map<Integer,double[]>> dischargeData=null;
		
		//setup discharge info
		if(sd.getStreamFlowFile()!=null) {
			dischargeData=createDischargeData(sd.getStreamFlowFile(),hs);
		}
		List<HydrologicalUnit>units=new ArrayList<HydrologicalUnit>();
		//load the file here
		if(sd.getHydrologicalFile()!=null && sd.getHydrologicalFile().contains(".shp")){
			List<Map<String, Object>>hydroData=ShapeFileLoader.shpLoader(new File(sd.getHydrologicalFile()));
			Iterator<Map<String, Object>> ih = hydroData.iterator();
			while(ih.hasNext()){
				Map<String,Object>data = ih.next();
				Geometry geo = (Geometry)data.get("the_geom");
				
				double type =(Double)data.get("TYPE");
				ArrayList latLongs = new ArrayList();
				Coordinate[] coords=geo.getCoordinates();
				double size=1;
				Renderable shape=null;
				
				size=3;
				if(type==1.0)
					size=1;
				for(int i=0; i<coords.length;i++){
					Coordinate c = coords[i];	
					latLongs.add(Position.fromDegrees(c.y, c.x, 0));
				}
				shape = new Polyline(latLongs);
				((Polyline) shape).setAntiAliasHint(Polyline.ANTIALIAS_NICEST);
				
				((Polyline) shape).setColor(Color.BLUE);
				((Polyline) shape).setLineWidth(size);
				((Polyline) shape).setPathType(Polyline.LINEAR);
				((Polyline) shape).setFollowTerrain(true);
				SurfacePolyline sp = new SurfacePolyline(latLongs);
			
	/*			else{
					for(int i=0; i<coords.length;i++){
						Coordinate c = coords[i];
						latLongs.add(LatLon.fromDegrees(c.y, c.x));
					}					
					shape=new SurfacePolygon(latLongs,Color.BLUE,Color.BLUE);
				} */
				HydrologicalUnit hu = new HydrologicalUnit(geo,shape);
				hu.setName("canal");
				hs.add(hu);
				units.add(hu);
				geography.move(hu,geo);
				context.add(hu);
			}	
		}
		hs.setUnits(units);
		if(sd.isMunicipalSystem()){
			buildMunicipalSystem(hs,sd);
		}
		hydroNumber++;
		
		return hs;
	}
	
	public Map<String,Map<Integer,double[]>> createDischargeData(String file,HydrologicalSetting hs){
		List<Map>riverFlowData=CSVLoader.loadCSVStatic(file);
		Map<String, Map<Integer, double[]>> dischargeData = returnRiverDischargeData(riverFlowData);
		HydroBehavior bh=hs.getHydroBehavior();
		Iterator<String>ki = dischargeData.keySet().iterator();
		while(ki.hasNext()){
			String key = ki.next();
			Map<Integer,double[]> localData = dischargeData.get(key);
			Iterator<Integer>kii = localData.keySet().iterator();
			while(kii.hasNext()){
				int k = kii.next();
				bh.setDischargeInfo(key,k, localData);
			}
		}	
		return dischargeData;
	}
	
	/**
	 * Method to set rainfall data for salinization
	 * @param file the file to load data
	 * @return  
	 */
	public void createRainfallData(String file){
		List<Map>riverFlowData=CSVLoader.loadCSVStatic(file);
		Map<String, Map<Integer, double[]>> rainfallData = returnRiverDischargeData(riverFlowData);
		
		String area=(String)RunEnvironment.getInstance().getParameters().getValue("rainArea");
		
		String[] areas=area.split(",");
		Map<String,Map<Integer,double[]>>input=new HashMap<String,Map<Integer,double[]>>();
		
		for(String s: areas){
			Map<Integer,double[]> localData = rainfallData.get(s);
			input.put(s, localData);
		}
			
		SalinizationModel.seasonalRainfall=input;
	}
	
	/**
	 * Method to initialize remote sensing-based hydrological discharge model.
	 * @param hs the Hydrological setting
	 * @param hu the hydrological unit
	 * @param data the input data
	 */
	public void buildRemoteSensingHydroModel(HydrologicalSetting hs, HydrologicalUnit hu, Map<String,Object>data){
		Iterator<String> ic = data.keySet().iterator();
		
		double slope=((Double)data.get("slope")).doubleValue();
		double bankfullWidth=((Double)data.get("bankfullWidth")).doubleValue();
		
		hu.setSlope(slope);
		hu.setBankfullWidth(bankfullWidth);
		
		while(ic.hasNext()){
			String key = ic.next();
			if(key.contains("waterWidth")){
				String[]month=key.split("idth");
				double result=Double.parseDouble((String)data.get(key));
				Map<HydrologicalUnit,Map<Integer,Double>> wValues=
					((RemoteSensingEstimator)hs.getHydroBehavior()).getHydroloigcalWidth();
				
				Map<Integer, Double> iData=null;
				if(!wValues.containsKey(key)) 
					iData = new HashMap<Integer,Double>();
				else
					iData=wValues.get(hu);	
				
				iData.put(Integer.parseInt(month[1]), result);
				wValues.put(hu, iData);
			}
		}
		((RemoteSensingEstimator)hs.getHydroBehavior()).setHydrologicalSetting(hs);
	}
	
	/**
	 * Method for creating a municipal water system based on input data
	 * @param hs a hydrological setting object to add hydrological unit(s) to
	 */
	private void buildMunicipalSystem(HydrologicalSetting hs,ScenarioData sd){
		Iterator<Community> ic = communities.iterator();
		while(ic.hasNext()){
			Community c = ic.next();
			Map<String,Map<Integer,double[]>>dData=createDischargeData(sd.getMunicipalFile(),hs);
			Map<Integer,double[]>dataM=dData.get(c.getId());
			HydrologicalUnit hu = new HydrologicalUnit(c.getLocation(),null);
			hu.setMunicipalSource(true);
			hu.setName((String)c.getId()+" Municipal");
			dData.put(hu.getName(), dataM);
			dData.remove(c.getId());
			hs.getHydroBehavior().setData(dData);
			hs.add(hu);
		}
	}
	
	/**
	 * Method to get discharge data from file
	 * @param riverFlowData the discharge data loaded from a file
	 * @return a map with organized discharge data based on monthly discharge
	 */
	public Map<String,Map<Integer,double[]>> returnRiverDischargeData(List<Map>riverFlowData){
		
		if(hydroSourceData==null)
			hydroSourceData = new HashMap<String,Map<Integer,double[]>>();
		Map<String,Map>tempData = new HashMap<String,Map>();
		Iterator<Map> ri = riverFlowData.iterator();
		while(ri.hasNext()){
			Map data = ri.next();
			Map<Integer,List>temp = null;
			List<Double> monthData=null;
			String mV=(String)data.get("mean_va");
			double meanV =Double.parseDouble(mV);
			int month = Integer.parseInt((String)data.get("month_nu"))-1;
			if(!hydroSourceData.containsKey(data.get("Water Body"))){
					temp=new HashMap<Integer,List>();
					tempData.put((String) data.get("Water Body"), temp);
					hydroSourceData.put((String) data.get("Water Body"), null);
			}
			temp=tempData.get((String)data.get("Water Body"));
			if(temp.containsKey(month)){
				monthData = temp.get(month);
				monthData.add(meanV);
			}
			else{
				monthData=new ArrayList<Double>();
				monthData.add(meanV);
				temp.put(month, monthData);
			}
		}
		
		Iterator<String> ik=tempData.keySet().iterator();
		while(ik.hasNext()){
			String river = ik.next();
			Map<Integer,List>temps=tempData.get(river);
			Iterator<Integer>ikk=temps.keySet().iterator();
			Map<Integer,double[]> disch=new HashMap<Integer,double[]>();
			while(ikk.hasNext()){
				int key = ikk.next();
				List<Double>mData = temps.get(key);
				double[] data = new double[mData.size()];
				for(int i=0; i < mData.size(); i++){
					data[i]=mData.get(i);
				}
				disch.put(key, data);
			}
			hydroSourceData.put(river, disch);
		}
		return hydroSourceData;
	}	
	
	/**
	 * Method to create Organizations
	 * @param sd scenario data input
	 * @param model the model to associate with water use in the mine
	 * @param geography the geography the mine belongs to
	 * @return a list of organizations
	 */
}
