package ucl.salinization.dataLoad;

import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileReader;
import java.io.FileWriter;
import java.io.IOException;
import java.io.PrintWriter;
import java.util.ArrayList;
import java.util.List;

import simphony.util.messages.MessageCenter;
import ucl.salinization.data.ScenarioData;

import com.thoughtworks.xstream.XStream;
import com.thoughtworks.xstream.io.xml.DomDriver;

/**
 * Class to create an xml file to reference scenario data files
 * @author
 */
public class ScenarioXMLFileLoader {

	private List<ScenarioData>populations;  //list of scenario data for populations simulated
	
	//directory locations for data files
	public static String SocialShapeFilesDirectory = "data/SocialShapeFiles/";
	public static String HydrologyShapeFilesDirectory = "data/HydrologyShapeFiles/";
	public static String DemographicDataDirectory = "data/Demographics/";
	public static String MiningDataDirectory = "data/Mining/";
	public static String StreamFlowDataDirectory = "data/StreamFlowData/";
	public static String FieldDataDirectory = "data/FieldData/";
	public static String WaterUseModelData = "WaterUseModelData/";
	public static String HydroModelData="HydroModelData/";
	/**
	 * Default constructor
	 */
	public ScenarioXMLFileLoader(){
		populations=new ArrayList<ScenarioData>();
	}
	
	/**
	 * Method to setup narrative event data from an xml file
	 * @param file the file to load
	 */
	public static ScenarioXMLFileLoader SetupScenarioFileData(String file){
		XStream xstream = new XStream(new DomDriver());
		xstream.setClassLoader(ScenarioXMLFileLoader.class.getClassLoader());
		xstream.alias("Scenario",ScenarioXMLFileLoader.class);
		xstream.aliasType("ScenarioData", ScenarioData.class);
		ScenarioXMLFileLoader sf=null;
		try {
			sf=(ScenarioXMLFileLoader)xstream.fromXML(new FileReader(file));
			
		} catch (Exception e) {
			MessageCenter.getMessageCenter(ScenarioXMLFileLoader.class).error("Problems with reading" +
					"input XML file for Scenario", e);
			e.printStackTrace();	

		}
		return sf;
	}
	
	/**
	 * A static method that outputs file data into an xml file
	 * @param xml the xml file's data
	 * @param filename the name of the file
	 * @return an xml file output
	 */
	public static File Output(String xml,String filename){
		FileWriter fw= null;
		PrintWriter pw= null;

		// Open a file for output
	    System.out.println("writing output");
		
	    File file=null;
		try
		{
			file = new File(filename);
			fw= new FileWriter( file );
			pw=  new PrintWriter( fw );
			char[]seq=new char[]{'"','1','.','0','"'};
			String seqStr="";
			for(int i=0; i < seq.length; i++){
				seqStr+=seq[i];
			}
			
			char[]seqUTF=new char[]{'"','U','T','F','-','8','"'};
			String seqUTFStr="";
			for(int i=0; i < seqUTF.length; i++){
				seqUTFStr+=seqUTF[i];
			}
			pw.println("<?xml version="+seqStr+" encoding="+seqUTFStr+"?>");
			pw.print(xml);
			pw.close();
		}
		catch(IOException e){
			MessageCenter.getMessageCenter(ScenarioXMLFileLoader.class).error("Problems with creating" +
					"XML output", e);
		}
		return file;
	}
	
	/**
	 * Main method used to generate xml scenario file
	 * @param args
	 * @throws FileNotFoundException
	 * @throws IOException
	 */
/*	public static void main(String[] args) throws FileNotFoundException, IOException{
		XStream xstream = new XStream(new DomDriver());
		xstream.alias("Scenario",uaa.feral.util.loader.ScenarioXMLFileLoader.class);
		
		ScenarioXMLFileLoader sxf = new ScenarioXMLFileLoader();
		sxf.waterUseModel="SocialDistancingModel";
		sxf.hydrologyModel="VolumeEstimatorModel";
		
		ScenarioData sd = new ScenarioData("Pop1",ScenarioXMLFileLoader.SocialShapeFilesDirectory+"South_Central_Demographics.shp");
		sd.setHydrologicalFile(ScenarioXMLFileLoader.HydrologyShapeFilesDirectory+"NHDFlowlineProjected.shp");
		sd.setStreamFlowFile(ScenarioXMLFileLoader.StreamFlowDataDirectory+"KnikRiverPalmerDischarge.csv");
		sd.setDemographicFile(ScenarioXMLFileLoader.DemographicDataDirectory+"BirthandDeathfor2006.csv");
		sd.setWaterUseModelDataFile(ScenarioXMLFileLoader.WaterUseModelData+"SocialDistancingData.xml");
		sd.setHydroModelDataFile(ScenarioXMLFileLoader.HydroModelData+"HydroModelData.xml");
		sd.setMunicipalSystem(false);
		sd.setName("Palmer");
		sxf.populations.add(sd);
		
		xstream.setMode(XStream.XPATH_RELATIVE_REFERENCES);
		xstream.aliasType("ScenarioData", ScenarioData.class);
		String xml=xstream.toXML(sxf);
		String fileName= "ScenarioData.xml";
		String outputDirectory= "../FERAL/FERAL_Data/";
		
		String filename= outputDirectory+fileName;
		Output(xml,filename);
	}*/
	
	public static void main(String[] args) throws FileNotFoundException, IOException{
		XStream xstream = new XStream(new DomDriver());
		xstream.alias("Scenario",ucl.salinization.dataLoad.ScenarioXMLFileLoader.class);
		
		ScenarioXMLFileLoader sxf = new ScenarioXMLFileLoader();

		ScenarioData sd = new ScenarioData();
		sd.setMiningScenario(false);
		sd.setMunicipalSystem(false);
		sd.setName("Diyala Region");
		sd.setCommunityFile(ScenarioXMLFileLoader.SocialShapeFilesDirectory+"Sites.shp");
		sd.setStreamFlowFile(ScenarioXMLFileLoader.StreamFlowDataDirectory+"FishRiverDischarge.csv");
		sd.setFieldData(ScenarioXMLFileLoader.FieldDataDirectory+"Fields.shp");
		sd.setHydrologicalFile(ScenarioXMLFileLoader.HydrologyShapeFilesDirectory+
				"RiversAndCanals.shp");
		sxf.populations.add(sd);
		
		xstream.setMode(XStream.XPATH_RELATIVE_REFERENCES);
		xstream.aliasType("ScenarioData", ScenarioData.class);
		String xml=xstream.toXML(sxf);
		String fileName= "ScenarioData1.xml";
		String outputDirectory= "../Salinization/data/";
		
		String filename= outputDirectory+fileName;
		Output(xml,filename);
	}

	public List<ScenarioData> getPopulations() {
		return populations;
	}

	public void setPopulations(List<ScenarioData> populations) {
		this.populations = populations;
	}
}
