package ucl.salinization.displays;

import java.awt.Color;
import java.awt.event.ActionEvent;
import java.awt.event.MouseEvent;
import java.util.ArrayList;
import java.util.Collection;
import java.util.Iterator;
import java.util.List;

import javax.swing.AbstractAction;
import javax.swing.Action;
import javax.swing.JMenu;
import javax.swing.JOptionPane;
import javax.swing.JPopupMenu;

import repast.simphony.visualization.DisplayData;
import repast.simphony.visualization.Layout;
import repast.simphony.visualization.visualization2D.Display2D;
import repast.simphony.visualization.visualization2D.DisplayLayer2D;
import repast.simphony.visualization.visualization2D.IDisplayLayer2D;
import edu.umd.cs.piccolo.PCamera;
import edu.umd.cs.piccolo.PNode;
import edu.umd.cs.piccolo.event.PBasicInputEventHandler;
import edu.umd.cs.piccolo.event.PInputEvent;
import edu.umd.cs.piccolo.event.PInputEventListener;
import edu.umd.cs.piccolo.util.PPaintContext;
import edu.umd.cs.piccolo.util.PPickPath;

/**
 * A custom 2D display that adds pop-up menus to objects.
 * 
 * @author 
 *
 */
public class PopupMenuDisplay extends Display2D {

//	/**
//	 * Map of registered class pop-up menu items.
//	 */
//	private Map<Class<?>,Map<String,String>> menuClassMap;
	
	/**
	 * Pop-up menu constructor.
	 * @param data the display data
	 * @param layout the layout object
	 */
	public PopupMenuDisplay(DisplayData<?> data, Layout layout) {
		super(data, layout);
		
		addPopupEventHandler();
		
		// override the default repast rendering quality to use HQ rendering.
		canvas.setDefaultRenderQuality(PPaintContext.HIGH_QUALITY_RENDERING);
		
		setBackgroundColor(Color.black);
		
//		menuClassMap =  new HashMap<Class<?>, Map<String,String>>();
	}

	/**
	 * Creates the event handler for single-click mouse events which may optionally
	 * trigger pop-up menu events for registered classes.
	 */
	private void addPopupEventHandler() {
		PInputEventListener listener = new PBasicInputEventHandler() {
    	
		//when the mouse is clicked, the following events take place
			public void mouseClicked(PInputEvent pInputEvent) {
				if (pInputEvent.isLeftMouseButton() && pInputEvent.getClickCount() == 1) {
					PPickPath path = pInputEvent.getPath();
					MouseEvent evt = (MouseEvent)pInputEvent.getSourceSwingEvent();
          
					executePopupEvent(path, evt);
				}
			}
		};
		canvas.addInputEventListener(listener);
  }

  /**
   * Finds objects matching picked paths and creates a pop-up menu for them.
   * 
   * @param path the picked path.
   * @param evt the MouseEvent that triggered the pick event.
   */
  private void executePopupEvent(PPickPath path, MouseEvent evt) {
    List<PNode> pNodes = path.getNodeStackReference();
    
    // remove all the non-representational nodes from the list
    for (Iterator<PNode> iter = pNodes.iterator(); iter.hasNext();) {
      PNode node = iter.next();
            
      if (node instanceof PCamera || node instanceof DisplayLayer2D) {
        iter.remove();
      }
    }

    // remove the valueDisplayLayer if contained in the probe list since it
    //  doesn't contain an object for which a method can be invoked.
    if (pNodes.contains(valueDisplayLayer)){
    	pNodes.remove(valueDisplayLayer);
    }
    
    int numToFind = pNodes.size();

    List<Object> probedObjects = new ArrayList<Object>();
 
    Collection<IDisplayLayer2D> layers = classStyleMap.values();
    for (IDisplayLayer2D layer : layers) {
      Collection<Object> objs = layer.findObjsForItems(pNodes);
      probedObjects.addAll(objs);
      if (probedObjects.size() == numToFind) {
        // found all so no need to search additional layers
        break;
      }
      for (Object obj : objs) {
        // remove the ones we've found from the list to search
        pNodes.remove(layer.getVisualItem(obj));
      }
    }

    if (pNodes.size() != 0) {
      // more to find, so search for edges
      Collection<IDisplayLayer2D> netLayers = objectStyleMap.values();
      for (IDisplayLayer2D layer : netLayers) {
        Collection<Object> objs = layer.findObjsForItems(pNodes);
        probedObjects.addAll(objs);
        if (probedObjects.size() == numToFind) {
          // found all so no need to search additional layers
          break;
        }
        for (Object obj : objs) {
          // remove the ones we've found from the list to search
          pNodes.remove(layer.getVisualItem(obj));
        }
      }
    }

    for (Object o : probedObjects){
    	showPopupMenu(o,evt);
    }
  }
  
  /**
   * Create and show the pop-up menu for the object and event.
   * 
   * @param o the picked object.
   * @param evt the pick event.
   */
  private void showPopupMenu(Object o, MouseEvent evt) {
  	JPopupMenu menu = new JPopupMenu();
  	
/*  	if (o instanceof Step){
  		Step node = (Step)o;
  		
  		for (Object a : node.getAgentLayer(Actor.class)){
  			Actor actor = (Actor)a; 
  			
  			// dont't put Factions on the display list
  			if (actor instanceof Faction) continue;
  			
  			boolean enabled = true;
  			
  			JMenu submenu = new JMenu("Branch " + actor);
  			menu.add(submenu);
  			
  			// Don't allow to branch the same actor consecutively or if already branched.
  			if ((node.getActorID() != null && 
  					node.getActorID().equals(actor.getName())) ||
  					node.isBranched())
  				
  				submenu.setEnabled(false);
  			
  			for (int i=1; i<=8; i++)
  				submenu.add((Action) new ScenarioAction((Step)o, actor.getName(), i));
  		} */
  		
  		//add menu action items
//  	menu.add((Action) new ShowMapAction((Step)o));
// 		menu.add((Action) new ShowHighlightAction((Step)o));
//  	menu.add((Action) new ShowIterateAction((Step)o));
  		
  		menu.show(this.getPanel(), evt.getX(), evt.getY());
//  	}
	}
  
  /**
   * Action item to show the map
   * @author Eric Tatara
   *
   */
  private class ShowMapAction extends AbstractAction{
  		
	  	/**The game step*/
//	  	Step step;
	  	
	  	/**
	 * 
	 */
	private static final long serialVersionUID = 1L;

		/**
	  	 * Show map action constructor
	  	 * @param step a simulation game step
	  	 */
  //		public ShowMapAction(Step step){
 // 			super("Show Map");
  //			this.step = step;
 // 		}
  	
		public void actionPerformed(ActionEvent arg0) {
//			new DisplayCreator().createDisplay(step, DisplayCreator.DisplayType.GIS);
	  	
			//@TODO assign values to map proxy step.
			
		}  	
  }
  
  /**
   * 
   * @author maltaweel
   *
   */
  private class ShowHighlightAction extends AbstractAction{
//  	Step step;
 /* 	public ShowHighlightAction(Step step){
  		super("Highlight path");
  		this.step = step;
  	} */
  	
		public void actionPerformed(ActionEvent arg0) {
			
/*			for (Step s : step.getPath())
			  s.setHighlighted(true); */
			
			render();
		}  	
  }
  
  private class ShowIterateAction extends AbstractAction{
 // 	Step step;
 /* 	public ShowIterateAction(Step step){
  		super("Iterate this step");
  		this.step = step;
  	} */
  	
		public void actionPerformed(ActionEvent arg0) {

			String s = JOptionPane.showInputDialog("Please input number of iterations"); 
			
//			step.iterateSteps(Integer.valueOf(s));
		}  	
  }
	
	/**
	 * Action for ...
	 *
	 */
	private class ScenarioAction extends AbstractAction {
//		Step step;
		String actorID;
		int ply;
		
/*		public ScenarioAction(Step step, String actorID, int ply){
		  super("Ply " + ply);
			this.step = step;
			this.actorID = actorID;
			this.ply = ply;
		} */
		
		public void actionPerformed(ActionEvent e) {
//			step.branch(actorID,ply,null);
		  
		  // need to force update and render since this display is not registered
			// with the Repast GUI runner.
//		  update();
//		  render();
		}
	}
	
}