package ucl.salinization.model.landuse;

import java.util.Map;

import ucl.salinization.data.ScenarioData;


/**
 * Interface for running different hydrological behaviors
 * @author 
 */

public interface HydroBehavior {

	/**
	 * Method to run the model
	 */
	public void runModel();
	
	/**
	 * Method to get the current estimated discharge for rivers
	 * @return
	 */
	public Map<String,Double> getWaterQuantityByBody();
	
	/**
	 * Method gets data for a water body based on the water body's name, with data containing info on monthly discharge/volume
	 * @return a map with the water body's data
	 */
	public Map<String,Map<Integer,double[]>> getWaterBodyData();
	
	/**
	 * Setter for monthly discharge data
	 * @param name of the river this is relevant for
	 * @param time the time data is saved for (e.g. month, day, etc.)
	 * @param sample the discharge data associated with the month
	 */
	public void setDischargeInfo(String river, int time,Map<Integer,double[]> sample);
	
	/**
	 * Discharge/volume data set according to river/water source name and data arranged by month (month represented by an integer
	 * @param data the data map containing discharge/volume information
	 */
	public void setData(Map<String,Map<Integer,double[]>> data);
	
	/**
	 * Method to reset when a model should reset to older data. This is intended to be used in a Markov model
	 * @param tickReset tick when to reset
	 */
	public void setTickReset(double tickReset);
	
	/**
	 * Method to get the name of the model
	 */
	public String getName();
	
	/**
	 * Check to see if hydrology quantity is degraded
	 * @return a boolean to see if degrade quantity or not
	 */
	public boolean isDegradeQuantity();
	
	/**
	 * Method to set the scenario data
	 * @param scenarioData the scenario data
	 */
	public void setScenarioData(ScenarioData scenarioData);
}
