package ucl.salinization.model.physical;

import java.util.Calendar;
import java.util.GregorianCalendar;
import java.util.HashMap;
import java.util.Map;

import repast.simphony.engine.environment.RunEnvironment;
import ucl.salinization.data.HydrologyModelData;
import ucl.salinization.data.ScenarioData;
import ucl.salinization.model.landuse.HydroBehavior;

/**
 * Abstract class for hydrological behaviors evolving the hydrological unit
 * @author
 *
 */
public abstract class EvolveHydrology implements HydroBehavior{
	
	
	protected Map<String,Double>dischargeByRivers;
	protected Map<String, Map<Integer, double[]>> source;
	protected HydrologyModelData hmd;
	protected double daysPerTick;
	protected String name;
	protected boolean degradeQuantity;
	protected double degradeAmount=1;
	protected int count=0;
	protected ScenarioData scenarioData;
	
	public EvolveHydrology(){
		dischargeByRivers=new HashMap<String,Double>();
	}
	
	public void runModel(){
		
	}
	
	public String getName(){
		return name;
	}
	
	protected int getCalendarMonth(){
		Calendar cal = new GregorianCalendar();
		double tick=
			RunEnvironment.getInstance().getCurrentSchedule().getTickCount();
	
		double day=daysPerTick*tick;
		if(day>365.25)
			day=day%365.25;
		cal.set(Calendar.DAY_OF_YEAR, (int)day);
		int month=cal.get(Calendar.MONTH);
		
		return month;
	}
	
	protected boolean withinOneWeek(){
		Calendar cal = new GregorianCalendar();
		double tick=
			RunEnvironment.getInstance().getCurrentSchedule().getTickCount();
	
		double day=daysPerTick*tick;
		if(day>365.25)
			day=day%365.25;
		cal.set(Calendar.DAY_OF_YEAR, (int)day);
		int month=cal.get(Calendar.MONTH);
		cal.clear();
		cal.set(Calendar.DAY_OF_YEAR, (int)day+7);
		if(cal.get(Calendar.MONTH)!=month)
			return true;
		else
			return false;
	}
	
	public void setupModelData(){
		HydrologyModelData hmd=new HydrologyModelData();
		this.hmd=hmd.setupModelData(scenarioData.getHydroModelDataFile());
		if (hmd.isDegradeSystem())
			this.degradeAmount=1-hmd.getDegradeQuantity();
	}
	
	public Map<String,Double>getWaterQuantityByBody(){
		return dischargeByRivers;
	}
	
	public void setData(Map<String,Map<Integer,double[]>> data){
		this.source=data;
	}

	public Map<String,Map<Integer,double[]>> getWaterBodyData(){
		return source;
	}
	
	public void setTickReset(double tickReset){
		this.hmd.setTickReset(tickReset);
	}
	
	public static HydroBehavior CreateHydroModel(ScenarioData sd,String modelType){
		HydroBehavior hb=null;
		
		if(modelType.equals("VolumeEstimatorModel")){
			hb=new VolumeEstimatorModel();
		}
		
		hb.setScenarioData(sd);
		return hb;
	}

	public boolean isDegradeQuantity() {
		return degradeQuantity;
	}

	public void setDegradeQuantity(boolean degradeQuantity) {
		this.degradeQuantity = degradeQuantity;
	}

	public ScenarioData getScenarioData() {
		return scenarioData;
	}

	public void setScenarioData(ScenarioData scenarioData) {
		this.scenarioData = scenarioData;
	}

	public double getDegradeAmount() {
		return degradeAmount;
	}

	public void setDegradeAmount(double degradeAmount) {
		this.degradeAmount = degradeAmount;
	}
}
