package ucl.salinization.model.physical;

import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;
import java.util.List;
import java.util.ArrayList;

import repast.simphony.engine.environment.RunEnvironment;
import repast.simphony.parameter.Parameters;
import repast.simphony.random.RandomHelper;
import ucl.salinization.util.Statistics;


import cern.jet.random.Normal;
import cern.jet.random.Uniform;

/**
 * Model to use Markov Chain Monte Carlo Method (Metropolis-Hastings algorithm)
 * @author
 */
public class VolumeEstimatorModel extends EvolveHydrology{
	
//	Map<Object,double[]> sample;
	List<HydrologicalUnit> units;
	Normal distribution;
	Uniform uniform;
	 
	private int currentMonth;
	private HashMap<String,Map<Integer,double[]>> historical;
	
	public VolumeEstimatorModel(String test){
		super();
		uniform=RandomHelper.createUniform();
	}
	
	/**
	 * Main constructor used for model.
	 */
	public VolumeEstimatorModel(){
		super();
		source=new HashMap<String,Map<Integer,double[]>>();
		units=new ArrayList<HydrologicalUnit>();
		Parameters p = RunEnvironment.getInstance().getParameters();
		uniform=RandomHelper.createUniform();
		daysPerTick=(Double)p.getValue("daysPerTick");
		name="VolumeEstimatorModel";
	}
	
	public void setDischargeInfo(String riverN,int month,Map<Integer,double[]> sample){
		source.put(riverN, sample);
	}
	
	/**
	 * Monthly data to get.
	 * @param sample the data to get.
	 * @return data input values.
	 */
	
	public double[] divideMonthlyInfo(Map<Integer,double[]> sample){
		double[] currentMonthData=sample.get(currentMonth);
		return currentMonthData;
	}
	
	/**
	 * Method that could be used to estimate rainfall as well as discharge based on Metropilis-Hasting algorithm.
	 * @param currentMonthData input values for markov.
	 * @return
	 */
	public double[] estimateDischargeValue(double[] currentMonthData){
		double mean=(Statistics.computeMean(currentMonthData))*this.getDegradeAmount();
		double deviation=Statistics.
			computeStandardDeviation(currentMonthData, mean);
		distribution=RandomHelper.createNormal(mean, deviation);
//		List<Double>flowL=new ArrayList<Double>();
//		for(int i=0; i < currentMonthData.length;i++){
//			flowL.add(currentMonthData[i]);
//		}
//		Collections.sort(flowL);
		double[] points = new double[5000];
		points=metrop(points.length,deviation,points,mean);
		
		return points;
	}
	
	/**
	 * This is the main metropolis-hastings method.
	 * @param np
	 * @param alphap alpha input
	 * @param vec vector input for makov chain
	 * @param mean mean value
	 * @return
	 */
	public double[] metrop(int np, double alphap, double[] vec,double mean){
	  int n;
	  int i;
	  double alpha,x,can,u,aprob;
	  n=np;
	  alpha=alphap;
	  x=0.0;
	  vec[0]=x;
	  for (i=1;i<n;i++)
	    {
	      can=x+uniform.nextDoubleFromTo(-alpha,alpha);
	      aprob=Math.min(1.0, distribution.pdf(can)/distribution.pdf(x));
	      u=uniform.nextDouble();
	      if(!(aprob<=1)) {
	    	  vec[i]=mean+can;
	    	  continue;
	      }
	      if (u < aprob)
	    	  x=can;
	      
	      vec[i]=x;
	    }
	  return vec;
	}
	
	/**
	 * Method for creating original historical values that can later be used to reset markov process with original values
	 */
	private void createHistorical(){
		this.historical=new HashMap<String,Map<Integer,double[]>>();
		Iterator<String>si = source.keySet().iterator();
		while(si.hasNext()){
			String s = si.next();
			char[] a=s.toCharArray();
			String n = new String(a);
			Map<Integer,double[]> sample = source.get(s);
			Map<Integer,double[]> newSample=new HashMap<Integer,double[]>();
			Iterator<Integer>ii = sample.keySet().iterator();
			while(ii.hasNext()) {
				int i=ii.next();
				newSample.put(i, sample.get(i));
			}
			historical.put(n,newSample);
		}
	}
	
	/**
	 *Main method to run algorithm.
	 */
	public void runModel(){
		if(count==0) {
			this.setupModelData();
			createHistorical();
		}
		this.currentMonth=getCalendarMonth();
		if(distribution==null || uniform==null) {
			distribution=RandomHelper.getNormal();
			uniform=RandomHelper.createUniform();
		}
		Iterator<String> ic = source.keySet().iterator();
		while(ic.hasNext()){
			String riverN = ic.next();
			Map<Integer,double[]>sample=source.get(riverN);
			double currentMonthData[]=divideMonthlyInfo(sample);
			double currentDischarge[]=estimateDischargeValue(currentMonthData);
			if(count%this.hmd.getTickReset()!=0)
				sample.put(currentMonth, currentDischarge);
			else {  //the if-else clause is used to reset the discharge data so that data does not skew in only one direction
				Map<Integer,double[]> origSample=historical.get(riverN);
				for(int i=0; i<12;i++){
					sample.put(i, origSample.get(i));
				}
				source.put(riverN, sample);
			}
			double currentDischargeRate=0;
			if(withinOneWeek()){
				currentDischargeRate=refactorResults(sample,currentMonthData);
			}
			else
				currentDischargeRate=Statistics.computeMedian(currentDischarge, 0, currentDischarge.length-1);
	//		double currentDischargeRate=currentDischarge[currentDischarge.length-1];
	/*		if(currentDischargeRate<0){
				int i=currentDischarge.length-2;
				while(currentDischargeRate<0){
					currentDischargeRate=currentDischarge[i];
					i--;                                 
				}
			} */
			
			dischargeByRivers.put(riverN,currentDischargeRate);
		}
		count++;
	}
	
	/**
	 * Refactor data to adjust inputs for markov chain.
	 * @param sample the data set.
	 * @param currentMonthData the monthly input.
	 * @return
	 */
	public double refactorResults(Map<Integer,double[]> sample,double[] currentMonthData){
		int nextMonth=-11;
		if(currentMonth!=11)
			nextMonth=1;
		double[] nextMonthData=sample.get(currentMonth+nextMonth);
		if(currentMonthData.length-nextMonthData.length>100)
			nextMonthData=estimateDischargeValue(nextMonthData);
		double[] newData=Statistics.combineData(currentMonthData, nextMonthData);
		return Statistics.computeMean(newData,0,newData.length-1);
	}
}
