package ucl.salinization.social;

import gov.nasa.worldwind.geom.Angle;
import gov.nasa.worldwind.geom.Position;
import gov.nasa.worldwind.layers.IconLayer;
import gov.nasa.worldwind.render.Renderable;
import gov.nasa.worldwind.render.UserFacingIcon;

import java.awt.Dimension;
import java.util.ArrayList;
import java.util.Collection;
import java.util.Iterator;
import java.util.List;
import java.util.Map;

import com.vividsolutions.jts.geom.Point;

import repast.simphony.context.DefaultContext;
import repast.simphony.engine.environment.RunEnvironment;
import repast.simphony.engine.environment.RunState;
import repast.simphony.engine.schedule.IAction;
import repast.simphony.engine.schedule.ISchedule;
import repast.simphony.engine.schedule.ScheduleParameters;
import repast.simphony.engine.schedule.ScheduledMethod;
import repast.simphony.parameter.Parameters;
import repast.simphony.random.RandomHelper;
import repast.simphony.space.gis.Geography;
import repast.simphony.visualization.IDisplay;
import repast.simphony.visualization.gis3D.DisplayGIS3D;
import repast.simphony.visualization.gis3D.StyledMapLayer3D;
import repast.simphony.visualization.gisnew.AbstractSurfaceLayer;
import repast.simphony.visualization.gisnew.NewDisplayGIS;
import ucl.salinization.model.physical.Field;
import ucl.salinization.output.FieldOutput;
import ucl.salinization.output.Outputter;


/**
 * Community object that contains person agents.
 * The community does display agent behavior, in addition to containing person agents
 * @author
 *
 * @param <Person>
 */
public class Community  {

	
	Point location;  //location of the community
	
	
	private UserFacingIcon icon;  //visual icon of the community
	private double daysPerTick;
	private double tickYearly;
	private double importance;
	private int population;
	private List<Field>fields; //fields for the community;
	List<Person>people;


	private String id;
	
	public double getImportance() {
		return importance;
	}

	public void setImportance(double importance) {
		this.importance = importance;
	}

	public Community(){
		
	}
	
	/**
	 * Main constructor used to create a community
	 * @param id the community's id
	 * @param location the community's location
	 */
	public Community(String id,Point location){
		this.id=id;
		this.location=location;
		this.daysPerTick=0;
		this.tickYearly=0;
//		if(RunEnvironment.getInstance().getCurrentSchedule().getTickCount()>0)
//			init();
	}
	
	/**
	 * Class to run scheduled demographic functions
	 * @author Mark Altaweel
	 *
	 */
	private class RunDemographics implements IAction{
		public void execute() {
	//		runDemographics();
		}
	}
	
	private class ScheduleFields implements IAction{
		public void execute() {
			rotateFields();
		}
	}
	
	/**
	 * Class to run scheduled population manager in demographic functions
	 * @author Mark Altaweel
	 *
	 */
	private class ManageDemographics implements IAction{
		public void execute() {
	//		managePopulation();
		}
	}
	
	/**
	 * Class to run scheduled social model behaviors
	 * @author Mark Altaweel
	 *
	 */
	private class RunModels implements IAction{
		public void execute(){
	//		runModels();
		}
	}
	
	

	public Point getLocation() {
		return location;
	}

	public void setLocation(Point location) {
		this.location = location;
	}
	
	public void rotateFields(){
		double tick = RunEnvironment.getInstance().getCurrentSchedule().getTickCount();
		if(fields==null)
			return;
		for(Field f: fields){
			f.setPlantYear(f.getPlantYear()+1);	
		}
		Outputter.writeFileData(tick, fields);
		System.out.println(this.getId()+" : "+tick);
	}

	/**
	 * Method to initialize parameters, behaviors, and call visualization initialization
	 */
	@ScheduledMethod(start = 0, pick=ScheduledMethod.ALL, priority = ScheduleParameters.FIRST_PRIORITY)
	public void init(){
		 Parameters p = RunEnvironment.getInstance().getParameters(); 	 
		 
		 if(daysPerTick!=0 || this.tickYearly!=0)
			 return;
		 this.daysPerTick=(Double)p.getValue("daysPerTick");
		 this.tickYearly = (double) 365.25/daysPerTick;
		 		  
		 if(tickYearly>12)
				tickYearly=tickYearly/12;
			else
				tickYearly=1;
		
		 
		  ISchedule schedule2 = repast.simphony.engine.environment.RunEnvironment.getInstance()
		      	.getCurrentSchedule();
		  schedule2.schedule(ScheduleParameters.createOneTime(1, 0),new RunModels());
		  
		  if(!RunEnvironment.getInstance().isBatch())
			  initV();
		  
		  for(Person person: this.people){
			  if(fields==null)
				  break;
			  for(Field f : fields) {
				  if(f.getPerson()==null){
					  f.setPerson(person);
					  double r=RandomHelper.nextDouble();
					  if(r<.5){
						  f.setPlantYear(0);
						  f.setStatus(Field.PLANTED);
					  }
					  else
						  f.setPlantYear(1);
					  person.setField(f);
					  break;
				  }
			  }
		  }
		 
		  double daysPT=(Double)RunEnvironment.getInstance().getParameters().getValue("daysPerTick");
		  double tick=365.25/daysPT;
		  double startTick=213/daysPT;
		  
		  ISchedule schedule3 = repast.simphony.engine.environment.RunEnvironment.getInstance()
          .getCurrentSchedule();
		  
		  schedule3.schedule(ScheduleParameters.createRepeating(startTick, tick, ScheduleParameters.LAST_PRIORITY),
              new ScheduleFields());
	}
	
	/**
	 * Method that initializes a community visual object (in animation mode)
	 */
	private void initV(){
			List<IDisplay> displays=RunState.getInstance().getGUIRegistry().getDisplays();
			Iterator<IDisplay> ic = displays.iterator();
			NewDisplayGIS gis=null;
			while(ic.hasNext()){
				IDisplay display = ic.next();
				if(display instanceof NewDisplayGIS)
					gis=(NewDisplayGIS)display;
			}
			Map<Class, AbstractSurfaceLayer> objs=gis.getClassStyleMap();
			AbstractSurfaceLayer layer = objs.get(Community.class);
			Collection<Renderable> renderables=(Collection<Renderable>)layer.getRenderables();
			Iterator<Renderable> ico=layer.getRenderables().iterator();
		//	while(ico.hasNext()){
		//		Renderable r = ico.next();
		//		if(layer.getShapeToObjectMap().get(r) == this){
					
		//		}
		//	}
			UserFacingIcon icon = new UserFacingIcon("../Salinization/img/houses_icon.gif",
		              new Position(Angle.fromDegreesLatitude(location.getCentroid().getY()),
		            		  Angle.fromDegrees(location.getCentroid().getX()), 0));
            IconLayer layer2 = new IconLayer();
           
            if(this.population<=10)
            	icon.setSize(new Dimension(25, 25));
            else
            	icon.setSize(new Dimension(50, 50));
            this.icon=icon;
            layer2.addIcon(icon);
			gis.getWwglCanvas().getModel().getLayers().add(layer2);
//			gis.getWwglCanvas().redrawNow();
//			PlaceNameLayer l = new PlaceNameLayer(new PlaceNameServiceSet());
//			gis.getWwglCanvas().getModel().getLayers().add(l);	
	}
	

	public UserFacingIcon getIcon() {
		return icon;
	}

	public void setIcon(UserFacingIcon icon) {
		this.icon = icon;
	}

	public int getPopulation() {
		return population;
	}

	public void setPopulation(int population) {
		this.population = population;
	}

	public List<Field> getFields() {
		return fields;
	}

	public void setFields(List<Field> fields) {
		this.fields = fields;
	}

	public List<Person> getPeople() {
		return people;
	}

	public void setPeople(List<Person> people) {
		this.people = people;
	}

	public String getId() {
		return id;
	}

	public void setId(String id) {
		this.id = id;
	}
	
	public void addAll(List<Person> person){
		if(people==null)
			people=new ArrayList<Person>();
		people.addAll(person);
	}
	
}
