package ucl.salinization.util;

import com.vividsolutions.jts.geom.Coordinate;


/**
 * Class intended for simple calculations of spatial and hydrology data
 * @author 
 */
public class SpatialHydrologyCalculations {

	/**
	 * Method to calculate 3D distance using lat and long locations
	 * @param beginning the beginning node
	 * @param end the end node
	 * @return a double distance value
	 */
	public static double LatLongDistance(Coordinate beginning, 
			Coordinate end) {
		
		double R = 6371; // km
		double dLat = Math.toRadians(beginning.y-end.y);
		double dLon = Math.toRadians(beginning.x-end.x); 
		double a = Math.sin(dLat/2) * Math.sin(dLat/2) +
		        Math.cos(end.y) * Math.cos(beginning.y) * 
		        Math.sin(dLon/2) * Math.sin(dLon/2); 
		double c = 2 * Math.atan2(Math.sqrt(a), Math.sqrt(1-a)); 
		double distance = (R * c)*1000;  //units in meters
	
		return distance;
	}
	
	/**
	 * Method to calculate 3D distance from 2 coordinates
	 * @param beginning the beginning coordinate
	 * @param end the end coordinate
	 * @return a 3D distance
	 */
	public static double Distance3D(Coordinate beginning,Coordinate end){
		double xBeg=beginning.x;
		double yBeg=beginning.y;
		double zBeg=beginning.z;
		if(Double.isNaN(zBeg))
			zBeg=0;
		double xEnd=end.x;
		double yEnd=end.y;
		double zEnd=end.z;
		if(Double.isNaN(zEnd))
			zEnd=0;
		double xDifference=xEnd-xBeg;
		double yDifference=yEnd-yBeg;
		double zDifference=zEnd-zBeg;
	
		double distance=Math.sqrt(Math.pow(xDifference, 2) 
			+ Math.pow(yDifference, 2)+Math.pow(zDifference, 2));
		
		return distance;
	}
	
	/**
	 * Volume is calculated over a full day period using discharge (m^3/sec)
	 * @param discharge the discharge of water per second
	 * @return the volume of water in a given day
	 */
	public static double DailyVolumeCalculator(double discharge){
		double volume=0d;
		
		volume=discharge*24*60*60;
		
		return volume;
	}

}
